@@ -1,14 +1,8 @@
 checkEarthdataLogin = function(
   method = "auto"
-  , server = c("LPDAAC", "LAADS")
   , path = "~/.netrc"
 ) {
   
-  server = match.arg(
-    server
-    , several.ok = TRUE
-  )
-  
   ## construct online file paths
   remote_urls = genString(
     "MCD64A1"
